/************************
*   Block device interface
*  Copyright Pu Bo 2001
*/

#ifndef _BLOCKDEVINF_H
#define _BLOCKDEVINF_H

#include "MemPool.h"

/* interface id */
#define IID_BLOCK_DEV LONG_ID('B','L','K','D')
#define MAX_BLK_DEV_NAME 11

typedef MemHndl BlockDeviceHndl;


typedef unsigned short BlkNum;

typedef unsigned short BlkSize;

typedef enum {
    kBDNoErr,
    kBDErrStorageNotAvailable,
    kBDErrWriteStorage,
    kBDErrReadStorage,
    kBDErrDevAlreadyExist,
    kBDErrDevNonExist,
    kBDErrBlockNotFound,
    KBDErrNotImplemented,
    kBDErrUnsupportedBlockSize,
    KBDErrMem,
    kBDErrWrongName,
    kBDErrIncompatibleDev

}BDErr;

/* interface */
typedef struct {
    BlockDeviceHndl (*CreateBlockDevice)(const char far *deviceName,
            BlkNum numOfBlocks,BlkSize blockSize);
    BlockDeviceHndl (*OpenBlockDevice)(const char far *deviceName);
    BlkNum (*GetTotalBlocks)(BlockDeviceHndl blockDevH);
    BlkSize (*GetBlockSize)(BlockDeviceHndl blockDevH);
    BlkSize (*WriteBlock)(BlockDeviceHndl blockDevH,BlkNum blockIdx,
            MemHndl bufH,BlkSize numBytes);
    BlkSize (*ReadBlock)(BlockDeviceHndl blockDevH,BlkNum blockIdx,
            MemHndl bufH,BlkSize numBytes);
    BlkSize (*WriteBlockOffset)(BlockDeviceHndl blockDevH,
            BlkNum blockIdx,
            BlkSize offset,
            MemHndl bufH,
            MemSize bufOffset,
            BlkSize numBytes);
    BlkSize (*ReadBlockOffset)(BlockDeviceHndl blockDevH,
            BlkNum blockIdx,
            BlkSize offset,
            MemHndl bufH,
            MemSize bufOffset,
            BlkSize numBytes);
    BDErr (*IncreaseBlockDev)(BlockDeviceHndl blockDevH,BlkNum numBlocks);
    BDErr (*DecreaseBlockDev)(BlockDeviceHndl blockDevH,BlkNum numBlocks);
    BDErr (*CloseBlockDevice)(BlockDeviceHndl blockDevH);
    BDErr (*GetLastError)();
}BlockDeviceInterface;

typedef BlockDeviceInterface far * BlkDevIntfPtr;

#endif