/* File System Interface
 * Copyright 2001 Pu Bo */
#ifndef _FILESYSIFC_H
#define _FILESYSIFC_H

#include "MemPool.h"
#ifdef WIN32
#include <stdio.h>
#else
#include "define.h"

/* definitions */
#define IID_FILE_SYS LONG_ID('F','S','Y','S')
#endif

#define MAX_FILE_NAME 32

#ifdef WIN32
typedef FILE * FileHndl;
#define PATH_DELIMITOR '\\'
#else
typedef MemHndl FileHndl;
typedef MemHndl FileSysHndl;
#endif

typedef enum {
	kFSNoErr,
	kFSErrInvalidFS,
	kFSErrMem,
	kFSErrFileNotFound,
	kFSErrReadFile,
	kFSBadFileType,
	kFSErrWriteFile,
	kFSErrCreateFile,
	kFSErrNotImplemented,
	kFSErrSeekFile,
	kFSErrDelFile,
	kFSErrFileAlreadyExist,
	kFSErrDirNotEmpty,
	kFSErrBadFileName
}FSErr;

typedef enum {
	kFSReadOnly,
	kFSWriteOnly,
	kFSReadWrite
}FileOpenMode;

#ifndef WIN32
/* interface */
typedef struct {
	FileSysHndl (*OpenFileSys)(const char far *fileSysName);
	FSErr (*ChangeCurDir)(FileSysHndl fileSysH,const char far *pathName);
	unsigned short (*GetDirFileNumber)(FileHndl dir);
	void (*GetFileName)(FileHndl dir,unsigned short fileIdx,char far *fileName);
	bool (*IsDir)(FileHndl dir,unsigned short fileIdx);
	bool (*IsFileDir)(FileHndl fileH);
	FileHndl (*MakeDir)(FileSysHndl fileSysH,const char far *name);
	FileHndl (*FileOpen)(FileSysHndl fileSysH,const char far *name,FileOpenMode mode);
	unsigned long (*FileRead)(FileHndl fileH,MemHndl bufH,unsigned long size);
	unsigned long (*FileWrite)(FileHndl fileH,MemHndl bufH,unsigned long size);
	FSErr (*FileSeek)(FileHndl fileH,unsigned long pos);
	unsigned long (*FileTell)(FileHndl fileH);
	unsigned long (*GetFileSize)(FileHndl fileH);
	void (*FileClose)(FileHndl fileH);
	FSErr (*DeleteFile)(FileSysHndl fileSysH,const char far *pathName);
	void (*CloseFileSys)(FileSysHndl fileSysH);
	FSErr (*GetLastError)();
}FileSysInterface;

typedef FileSysInterface far * FileSysIntfPtr;
#endif

#endif