/* Index Node interface
 *
 * INode system
 *
 * Copyright Pu Bo 2001 */
#ifndef _INODEIFC_H
#define _INODEIFC_H

#include "MemMgr.h"
#include "BlkDevIfc.h"

/* definitions */
#define IID_INODE LONG_ID('I','N','O','D')
#define INODE_MODE_UNUSED 0
#define INODE_MODE_DIR 1
#define INODE_MODE_FILE 2
#define INODE_ALLOCATED 0xff
#define MIN_BLOCK_SIZE 512
#define MIN_NUM_BLOCKS 2
#define ROOT_INODE_ID 0
#define INVALID_INODE_ID 0xffff

typedef enum {
    kINodeNoErr,
    kINodeErrPosOusideRange,
    kINodeErrBlkNotFound,
    kINodeErrReadBlk,
    kINodeErrWriteBlk,
    kINodeErrNodeNotFound,
    kINodeErrFreeBlk
}INodeErr;

typedef MemHndl INodeSysHndl;
typedef MemHndl INodeHndl;
typedef unsigned short INodeId;

/* interface */
typedef struct {
    INodeSysHndl (*CreateINodeSys)(BlkDevIntfPtr blkIfcP,const char far *name,
            BlkNum numOfBlocks,BlkSize blockSize);
    INodeSysHndl (*OpenINodeSys)(BlkDevIntfPtr blkIfcP,const char far *name);
    void (*CloseINodeSys)(INodeSysHndl iNodeSysH);
    /* INode access */
    INodeId (*AllocINode)(INodeSysHndl iNodeSysH);
    INodeHndl (*OpenINode)(INodeSysHndl iNodeSysH,INodeId nodeId);
    INodeErr (*SeekINode)(INodeHndl nodeH, unsigned long pos);
    unsigned long (*TellINode)(INodeHndl nodeH);
    unsigned long (*GetINodeSize)(INodeHndl nodeH);
    void (*TrimINode)(INodeHndl nodeH);
    unsigned char (*GetINodeMode)(INodeHndl nodeH);
    bool (*SetINodeMode)(INodeHndl nodeH,unsigned char mode);
    unsigned char (*GetINodeLink)(INodeHndl nodeH);
    bool (*SetINodeLink)(INodeHndl nodeH,unsigned char link);
    unsigned long (*ReadINode)(INodeHndl nodeH,MemHndl bufH,unsigned long size);
    unsigned long (*WriteINode)(INodeHndl nodeH,MemHndl bufH,unsigned long size);
    void (*CloseINode)(INodeHndl nodeH);
    INodeErr (*DelINode)(INodeSysHndl iNodeSysH,INodeId nodeId);
    INodeSysHndl (*GetINodeSys)(INodeHndl nodeH);
}INodeInterface;
typedef INodeInterface far * INodeIfcPtr;

#endif