/**************************************************
** MemMgr lib Ver 1.2
** Manages memory for the application
**
** This is a wrapper of MemPool.lib, makes it easier
** to use. It create only 1 memory pool inside, during
** MemMgrInit(). So all of other modules can request memory
** from MemMgr.lib.
**
** It is recommended that the developer calculate the
** memory usage of all globle variables, and put the
** rest of memory under the MemMgr.lib's control.

    e.g.
    ....in main module...
    byte far memPool[ALL_MEM_LEFT];
    .....

    main()
    {
        .....Before any call to MemMgr.lib functions
        MemMgrInit(memPool,ALL_MEM_LEFT);
        ....
    }


** Copyright Pu Bo 2001, pubo@bigfoot.com
*/
#ifndef _MEMMGR_H
#define _MEMMGR_H

#ifdef WIN32
#include "CasioTyp.h"
#else
#ifdef PV3S1600
#include <stddef.h>
#else
#include "stdrom.h"
#endif
#endif
#include "MemPool.h"
/* defines */

#if defined(PV3S1600) || defined(WIN32)
typedef struct {
    MemHndl mH;
    MemSize mOffset;
}MemOffset;
#else
typedef unsigned long MemOffset;
#endif

#ifdef WIN32
inline MemOffset GET_MEM_OFFSET(MemHndl memH,MemSize offset)
{
    MemOffset mo;
    mo.mH = memH;
    mo.mOffset = offset;
    return (mo);
}

typedef struct {
    MemPtr mPtr;
    MemSize mSize;
}MemRec;

inline MemPoolErr MemMgrInit(MemPtr memAddr,MemSize memSize) {
    return (kMPNoErr);
}

inline MemPoolErr MemMgrInitEx(MemPtr memAddr,MemSize memSize,unsigned short recNum) {
    return (kMPNoErr);
}

#define MemMgrAllocate(blockSize,memHndl) _MemMgrAllocate(blockSize,(MemHndl *)(memHndl))
inline MemPoolErr _MemMgrAllocate(MemSize blockSize,MemHndl *memHndl) {
    MemRec *mP = new MemRec[1];

    mP->mPtr = new byte[blockSize];
    mP->mSize = blockSize;

    *memHndl = (MemHndl)mP;

    return (kMPNoErr);
}

#define MemMgrFree(memHndl) _MemMgrFree((MemHndl)(memHndl))
inline MemPoolErr _MemMgrFree(MemHndl memHndl) {
    MemRec *mP = (MemRec *)memHndl;
    delete mP->mPtr;
    delete mP;

    return (kMPNoErr);
}

#define MemMgrChangeSize(memHndl,newSize) _MemMgrChangeSize((MemHndl)(memHndl),newSize)
inline  MemPoolErr _MemMgrChangeSize(MemHndl memHndl,MemSize newSize) {
    MemRec *mP = (MemRec *)memHndl;

    if (newSize <= mP->mSize) {
        mP->mSize = newSize;
        return (kMPNoErr);
    }

    char *newP = new char[newSize];
    memcpy(newP,mP->mPtr,mP->mSize);
    delete mP->mPtr;
    mP->mPtr = newP;
    mP->mSize = newSize;

    return (kMPNoErr);
}

#define MemMgrGetSize(memHndl) _MemMgrGetSize((MemHndl)(memHndl))
inline MemSize _MemMgrGetSize(MemHndl memHndl) {
    if (memHndl == NULL) {
        return (0);
    }

    MemRec *mP = (MemRec *)memHndl;

    return (mP->mSize);
}

inline MemPtr MemMgrGetMemPtr(MemOffset offset) {
    return ((char *)(*offset.mH)+offset.mOffset);
}

inline MemHndl MemMgrGetMemHndl(const MemPtr ptr) {
    return (NULL);
}

#else

#ifdef PV3S1600
MemOffset GET_MEM_OFFSET(MemHndl memH,MemSize offset);
#else
#define GET_MEM_OFFSET(memH,offset) (((unsigned long)FP_OFF(memH)<<16)|((MemSize)(offset)))
#endif

/* Functions */
MemPoolErr MemMgrInit(MemPtr memAddr,MemSize memSize);
MemPoolErr MemMgrInitEx(MemPtr memAddr,MemSize memSize,unsigned short recNum);
MemPoolErr MemMgrAllocate(MemSize blockSize,MemHndl *memHndl);
MemPoolErr MemMgrFree(MemHndl memHndl);
MemPoolErr MemMgrChangeSize(MemHndl memHndl,MemSize newSize);
MemSize MemMgrGetSize(MemHndl memHndl);
MemPtr MemMgrGetMemPtr(MemOffset offset);
MemHndl MemMgrGetMemHndl(const MemPtr ptr);
#endif
#endif