/**************************************************
** MemPool lib Ver 1.2
**
** Allocated memory must be used via the handle.
** It is not safe to keep the memory pointer, since
** the address may change during memory purge.
** Memory purge may happen in AllocateMemBlock()
** and ChangeMemBlockSize().
**
    e.g.
    typedef struct {
	 ....
	 ....
    }MyStruct;

    typedef MyStruct far * far * MyStructHndl;

    byte far memPool[20000];
    .....
    .....
    func()
    {
		MyStructHndl hndl;
		MemHndl otherH;
		MemPoolErr err;
		MemPoolRef poolRef;
		MyStruct *ptr;
		...
		...

		err = CreateMemPool(memPool,20000,&poolRef);
		...check the err...
		....
		err = AllocateMemBlock(poolRef,sizeof(MyStruct),&hndl);
		.....

		... to use it, via handle only ...
		... pointer is OK if you are sure no memory purge happen...
		(**hndl).member = ....
		ptr = *hndl;
		ptr->member.... This is OK because no memory purge happen
		................after the assignment ptr = *hndl
		......
		err = AllocateMemBlock(poolRef,1000,otherH);
		ptr->member....  This is WRONG, because memory purge may
		................happen in the above allocate.
		(**hndl).member = .... OK
		ptr = *hndl;
		ptr->member....... OK
	}

** Copyright Pu Bo 2001, pubo@bigfoot.com
*/
#ifndef _MEMPOOL_H
#define _MEMPOOL_H

/* defines */

/* typedef */
typedef enum {
	kMPNoErr,
	kMPNoMemInPool,
	kMPTooSmall,
	kMPBlockNotFound,
	kMPInvalidPool
}MemPoolErr;

typedef void far * far * MemHndl;
typedef void far * MemPtr;
typedef void far * MemPoolRef;

#if defined(PV3S1600) || defined(WIN32)
typedef unsigned long MemSize;
#else
typedef unsigned short MemSize;
#endif


#ifdef PV3S1600
#define ALIGN_ADDRESS(adr) (char *)(((unsigned long)(adr)+3) & 0xfffffffc)
#define ALIGN_ADDRESS_BEFORE(adr) (char *)((unsigned long)(adr) & 0xfffffffc)
#else
#define ALIGN_ADDRESS(adr) (char far *)(adr)
#define ALIGN_ADDRESS_BEFORE(adr) (char far *)(adr)
#endif

/* Functions */
MemPoolErr CreateMemPool(MemPtr poolAddr,MemSize poolSize,MemPoolRef *poolRef);
MemPoolErr CreateMemPoolEx(MemPtr poolAddr,MemSize poolSize,MemPoolRef *poolRef,unsigned short recNum);
MemPoolErr AllocateMemBlock(MemPoolRef thePool,MemSize memSize,MemHndl *blockHndl);
MemPoolErr FreeMemBlock(MemPoolRef thePool,MemHndl blockHndl);
MemPoolErr ChangeMemBlockSize(MemPoolRef thePool,MemHndl blockHndl,MemSize newSize);
MemSize GetMemBlockSize(MemPoolRef thePool,MemHndl blockHndl);
MemHndl GetHandleFromPtr(MemPoolRef thePool,const MemPtr ptr);

#endif